#!/usr/bin/env node

const chalk = require('chalk');
const commandLineArgs = require('command-line-args');
const commandLineUsage = require('command-line-usage');

const syscheck = require('./scripts/syscheck');
const deploy = require('./scripts/deploy');
const deployNext = require('./scripts/deployNext');
const deployLaravel = require('./scripts/deployLaravel');
const deployCraft = require('./scripts/deployCraft');

/* first - parse the main command */
const mainDefinitions = [
  {name: 'command', defaultOption: true, defaultValue: 'help'},
  {name: 'verbose', alias: 'v', type: Boolean, defaultValue: false},
  {
    name: 'projectFolder',
    alias: 'f',
    type: String,
    defaultValue: process.cwd(),
  },
  {name: 'siteName', alias: 's', type: String, defaultValue: ''},
  {name: 'siteRepo', alias: 'r', type: String, defaultValue: ''},
  {name: 'hostName', alias: 'h', type: String, defaultValue: ''},
  {name: 'branch', alias: 'b', type: String, defaultValue: false},
  {name: 'phpVersion', alias: 'p', type: String, defaultValue: ''},
  {name: 'nodeVersion', alias: 'n', type: String, defaultValue: ''},
  {name: 'webroot', alias: 'w', type: String, defaultValue: 'public'},
  {name: 'mysql', alias: 'q', type: String, defaultValue: ''},
];

const mainOptions = commandLineArgs(mainDefinitions, {
  stopAtFirstUnknown: true,
});
const helpOptions = commandLineArgs(mainDefinitions);
const argv = mainOptions._unknown || [];

const deployer = new deploy(mainOptions, __dirname);
const nextDeployer = new deployNext(mainOptions, __dirname);

switch (mainOptions.command.toLowerCase()) {
  case 'syscheck':
    syscheck.checkDependancies(mainOptions);
    break;
  case 'project':
    deployer.deployProject();
    break;
  case 'craft':
    const craftDeployer = new deployCraft(mainOptions, __dirname);
    craftDeployer.deployCraft();
    break;
  case 'composer':
    deployer.deployComposerCreate();
    break;
  case 'laravel':
    const laravelDeployer = new deployLaravel(mainOptions, __dirname);
    laravelDeployer.deployLaravel();
    break;
  case 'next':
  case 'nextjs':
  case 'react':
    nextDeployer.deploy();
    break;
  case 'help':
  default:
    const version = require('./package.json').version;
    const usage = commandLineUsage([
      {
        header: `Docksal Site Deployer v${version}`,
        content: 'Deploys dockerised site',
      },
      {
        header: 'Synopsis',
        content: '$ deploy <command> [options]',
      },
      {
        header: 'Command List',
        content: [
          {
            name: chalk.bold.yellow('help'),
            summary: 'Display help information',
          },
          {
            name: chalk.bold.yellow('syscheck'),
            summary: 'Checks that all the requirements are installed',
          },

          //   { name: chalk.bold.yellow("new"), summary: "Initialises a new generic container added to a git repo you checkoutout", },
          //    { name: chalk.bold.yellow('project'), summary: 'Checks out an existing version with docksal enabled already from git' },
          {
            name: chalk.bold.yellow('composer'),
            summary: 'Deploys a new composer create project instance',
          },
          {
            name: chalk.bold.yellow('laravel'),
            summary: 'Deploys a new laravel instance',
          },
          {
            name: chalk.bold.yellow('craft'),
            summary: 'Deploys a new craft instance',
          },
        ],
      },
      {
        header: 'Options',
        optionList: mainDefinitions,
      },
    ]);
    console.log(usage);

    break;
}
