var inquirer = require('inquirer');
const spawnAsync = require('@expo/spawn-async');
const fs = require('fs');
const util = require('util');
const exec = util.promisify(require('child_process').exec);
const render = require('./output');
const {exit} = require('process');

class Deploy {
  constructor(props, installedPath) {
    this.props = props;
    this.installedPath = installedPath;
  }

  async deployComposerCreate() {
    await this.checkSiteName();
    await this.getSitesPath();
    await this.checkSiteFolderStatus();
    await this.checkComposerProject();

    await this.createDocksalProjectFolder();
    await this.setupk8s();

    render.success(`Generic Composer Deployment Done: ${this.props.sitePath}`);
  }

  async addComposerDependancies(dependancies, dev = false) {
    const cli = await this.getDocksalCli();

    if (dev) {
      await this.runCliCommand(['composer', 'require', '--dev', ...dependancies, '--no-interaction', '-W']);
    } else {
      await this.runCliCommand(['composer', 'require', ...dependancies, '--no-interaction', '-W']);
    }
  }

  async runCliCommand(command, path = null) {
    const cli = await this.getDocksalCli();

    await this.runProcess('fin', ['rc', `--image=${cli}`, ...command], {
      cwd: path ?? this.props.sitePath,
    });
  }

  async createDocksalProjectFolder() {
    const cli = await this.getDocksalCli();
    const mysql = await this.getMySqlVersion();

    render.success(`Creating Project`);

    await this.runProcess(
      'fin',
      ['rc', `--image=${cli}`, 'composer', 'create-project', this.props.siteRepo, this.props.siteName, '--no-interaction', '--remove-vcs', '--prefer-dist'],
      {
        cwd: this.props.projectFolder,
      }
    );

    render.success(`Adding Composer Requirements for Server hosting (redis, gd, imagick)`);

    await this.runProcess(
      'sed',
      ['-i', '', 's:"require".*:"require" \\: {\\n"ext-redis"\\: "*",\\n"ext-gd"\\: "*",\\n"ext-imagick"\\: "*", :g', 'composer.json'],
      {
        cwd: this.props.sitePath,
      }
    );

    await this.runProcess('fin', ['config', 'generate', `--docroot=${this.props.webroot}`], {
      cwd: this.props.sitePath,
    });
    await this.runProcess('fin', ['config', 'set', `DOCKSAL_STACK=default`], {
      cwd: this.props.sitePath,
    });
    await this.runProcess('fin', ['config', 'set', `DB_IMAGE=docksal/db:mysql-${mysql}`], {
      cwd: this.props.sitePath,
    });
    await this.runProcess('fin', ['config', 'set', `CLI_IMAGE=${cli}`], {
      cwd: this.props.sitePath,
    });
    await this.runProcess('fin', ['config', 'set', `VIRTUAL_HOST=${this.props.hostName}.docksal.site`], {
      cwd: this.props.sitePath,
    });
    await this.runProcess('fin', ['config', 'set', 'COMPOSER_DEFAULT_VERSION=2'], {
      cwd: this.props.sitePath,
    });
    await this.runProcess('fin', ['mkcert', 'cr'], {
      cwd: this.props.sitePath,
    });

    // await exec(
    //   `echo "version: \"2.1\"" > ${this.props.sitePath}/.docksal/docksal.yml`
    // );
    await exec(`echo "services:" >> ${this.props.sitePath}/.docksal/docksal.yml`);
    await exec(`echo "  redis:" >> ${this.props.sitePath}/.docksal/docksal.yml`);
    await exec(`echo "    extends:" >> ${this.props.sitePath}/.docksal/docksal.yml`);
    await exec(`echo '      file: \${HOME}/.docksal/stacks/services.yml' >> ${this.props.sitePath}/.docksal/docksal.yml`);
    await exec(`echo "      service: redis" >> ${this.props.sitePath}/.docksal/docksal.yml`);
    await exec(`echo "  mail:" >> ${this.props.sitePath}/.docksal/docksal.yml`);
    await exec(`echo "    extends:" >> ${this.props.sitePath}/.docksal/docksal.yml`);
    await exec(`echo '      file: \${HOME}/.docksal/stacks/services.yml' >> ${this.props.sitePath}/.docksal/docksal.yml`);
    await exec(`echo "      service: mail" >> ${this.props.sitePath}/.docksal/docksal.yml`);
  }

  async setupk8s() {
    await exec(`cp -rf ${this.installedPath}/stubs/k8s/. ${this.props.sitePath}`);
  }

  async getSitesPath() {
    const result = await exec(`cd ${this.props.projectFolder} && pwd`);
    this.props.projectFolder = result.stdout.trim() + '/';
    this.props.sitePath = `${this.props.projectFolder}${this.props.siteName}`;
  }

  async checkSiteName() {
    if (this.props.siteName === '') {
      const {siteName} = await inquirer.prompt({
        type: 'propt',
        name: 'siteName',
        message: 'Please enter site name',
        validate: function(value) {
          var pass = value.match(/^[a-z0-9\-]*$/i);
          if (pass && value.length >= 3) {
            return true;
          }
          return 'Please enter a valid site name (a-z / 0-9)';
        },
      });
      this.props.siteName = siteName;
    }
    if (this.props.hostName === '') {
      const {hostName} = await inquirer.prompt({
        type: 'propt',
        name: 'hostName',
        message: 'Please enter site Hostname (?.docksal.site)',
        default: this.props.siteName,
        validate: function(value) {
          var pass = value.match(/^[a-z0-9\-\.]*$/i);
          if (pass && value.length >= 3) {
            return true;
          }
          return 'Please enter a valid site name (a-z / 0-9)';
        },
      });
      this.props.hostName = hostName.replace('.docksal.site', '').replace('.docksal', '');
    }
  }

  async checkSiteFolderStatus() {
    if (fs.existsSync(this.props.sitePath)) {
      const {install} = await inquirer.prompt({
        type: 'confirm',
        name: 'install',
        message: 'Folder Exists, Delete and restart site installation?',
        default: false,
      });

      if (install === false) {
        render.error('Installation failed due to existing site');
        process.exit(1);
      }
      await exec(`rm -r ${this.props.sitePath} &2>/dev/null`);
    }
  }

  async runProcess(cmd, args = [], options = {}, cb) {
    try {
      const calledCommand = spawnAsync(cmd, args, options);
      let childProcess = calledCommand.child;
      childProcess.stdout.on('data', data => {
        render.render(` ${data}`);
      });
      childProcess.stderr.on('data', data => {
        render.render(` ${data}`);
      });

      let result = await calledCommand;
      if (cb) {
        calledCommand.on('exit', () => {
          cb(result);
        });
      }
    } catch (error) {
      render.error(error);
      process.exit(1);
    }
  }

  async setPhpVersion() {
    if (this.props.phpVersion === '') {
      const {phpVersion} = await inquirer.prompt({
        type: 'list',
        name: 'phpVersion',
        message: 'Select PHP Version',
        choices: ['5.6', '7.1', '7.2', '7.3', '7.4', '8.0', '8.1'],
        default: '8.1',
      });

      this.props.phpVersion = phpVersion;
    }
  }

  async getMySqlVersion() {
    if (this.props.mysql === '') {
      const {mysql} = await inquirer.prompt({
        type: 'list',
        name: 'mysql',
        message: 'Select MySQL Version',
        choices: ['5.7', '8.0'],
        default: '8.0',
      });

      this.props.mysql = mysql;
    }

    return this.props.mysql;
  }

  async getDocksalCli() {
    await this.setPhpVersion();
    switch (this.props.phpVersion) {
      case '7.1':
      case '7.2':
      case '7.3':
      case '7.4':
        return `docksal/cli:stable-php${this.props.phpVersion}`;
      case '5.6':
      case '8.0':
      case '8.1':
        return `docksal/cli:php${this.props.phpVersion}`;
    }

    render.error('Failed to set a correct php version');
    throw 'failed';
  }

  async checkComposerProject() {
    if (this.props.siteRepo === '') {
      const {siteRepo} = await inquirer.prompt({
        type: 'propt',
        name: 'siteRepo',
        message: 'Please enter composer path',
        validate: function(value) {
          if (value.length >= 5) {
            return true;
          }
          return 'Please enter a valid site repository';
        },
      });
      this.props.siteRepo = siteRepo;
    }
  }
}

module.exports = Deploy;
