const Deploy = require('../deploy');
const addAuth = require('./addAuth');
const addExtras = require('./addExtras');
const render = require('../output');
const util = require('util');
const exec = util.promisify(require('child_process').exec);

class DeployLaravel extends Deploy {
  constructor(props, installedPath) {
    super();
    this.props = props;
    this.props.webroot = 'public';
    this.installedPath = installedPath;

    this.dependencies = [
      'spatie/laravel-permission',
      'predis/predis',
      'phpsa/laravel-api-controller',
      'doctrine/dbal',
      'custom-d/laravel-helpers',
      'aws/aws-sdk-php',
      'ranium/laravel-seedonce',
      'sentry/sentry-laravel',
    ];
    this.devDependencies = [
      'barryvdh/laravel-ide-helper',
      'phpsa/laravel-postman',
      'brianium/paratest',
      'nunomaduro/larastan',
      'stechstudio/laravel-php-cs-fixer',
    ];

    this.extras = ['larastan'];
    this.commands = [];

    if (this.props.siteRepo === '') {
      this.props.siteRepo = 'laravel/laravel';
    }
  }

  async deployLaravel() {
    await this.checkSiteName();
    await this.getSitesPath();
    await this.checkSiteFolderStatus();

    await this.composeLaravel();

    render.arrow(`Inititalising`);

    await this.runProcess('fin', ['setup'], {cwd: this.props.sitePath});

    render.success('Local Project Setup');

    render.arrow('Update your phpunit.xml file and make sure the test database is configured, and an APP_KEY is added to the vars');
    render.arrow('Run fin artisan test -p - confirm tests passing');
    render.arrow('Optionally - configure phpstan.neon.dist --- https://github.com/nunomaduro/larastan');
    render.arrow(
      'run fin composer phpstan -- and sort out any issues (should be about 5) -- or run fin exec vendor/bin/phpstan -b to generate a baseline to ignore these errors (not recomended)'
    );
    render.arrow(
      'you should now run fin artisan deploy:ci to setup the vars in Gitlab -- make sure to setup your repository first. you will need the id and a token.'
    );
  }

  async composeLaravel() {
    render.arrow(`Installing Laravel to ${this.props.sitePath}`);

    await this.addAuth();
    await this.addExtras();

    const cli = await this.getDocksalCli();

    await this.createDocksalProjectFolder();

    render.arrow('Setting .env variables settings...');

    this.runCliCommand;

    await this.runCliCommand(['sed', '-i', 's:DB_HOST=.*:DB_HOST=db:g', '.env']);
    await this.runCliCommand(['sed', '-i', 's:DB_DATABASE=.*:DB_DATABASE=default:g', '.env']);
    await this.runCliCommand(['sed', '-i', 's:DB_USERNAME=.*:DB_USERNAME=user:g', '.env']);
    await this.runCliCommand(['sed', '-i', 's:DB_PASSWORD=.*:DB_PASSWORD=user:g', '.env']);
    await this.runCliCommand(['sed', '-i', 's:REDIS_HOST=.*:REDIS_HOST=redis:g', '.env']);
    await this.runCliCommand(['sed', '-i', 's:CACHE_DRIVER=.*:CACHE_DRIVER=redis:g', '.env']);
    await this.runCliCommand(['sed', '-i', 's:SESSION_DRIVER=.*:SESSION_DRIVER=redis:g', '.env']);
    await this.runCliCommand(['sed', '-i', 's:QUEUE_CONNECTION=.*:QUEUE_CONNECTION=redis:g', '.env']);
    await this.runCliCommand(['sed', '-i', `s:APP_URL=.*:APP_URL=${this.props.hostName}.docksal.site:g`, '.env']);
    await this.runCliCommand(['sed', '-i', 's:MAIL_HOST=.*:MAIL_HOST=mail:g', '.env']);
    await this.runCliCommand(['sed', '-i', 's:MAIL_FROM_ADDRESS=.*:MAIL_FROM_ADDRESS=mail@customd.com:g', '.env']);

    await this.addComposerDependancies(this.dependencies);
    await this.addComposerDependancies(this.devDependencies, true);

    await this.runCliCommand(['php', 'artisan', 'vendor:publish', '--all']);

    await exec(`cat ${this.props.sitePath}/.env > ${this.props.sitePath}/.env.example`);

    await exec(`echo ".phpstorm.meta.php" >> ${this.props.sitePath}/.gitignore`);
    await exec(`echo "_ide_helper.php" >> ${this.props.sitePath}/.gitignore`);
    await exec(`echo "cghooks.lock" >> ${this.props.sitePath}/.gitignore`);

    await exec(`cp -rf ${this.installedPath}/stubs/laravel/. ${this.props.sitePath}`);

    await this.runCliCommand(['sed', '-i', 's:colors="true":colors="false":g', 'phpunit.xml']);
    // await this.runCliCommand([
    //   'sed',
    //   '-i',
    //   's:<env name="APP_ENV" value="testing"*:<env name="APP_ENV" value="testing" />\\\n<env name="APP_KEY" value="base64\:Sktp9UUnXyajBIjA2vDORL7xUOn7+Vtfcxe3xX+iquQ=" />:g',
    //   'phpunit.xml',
    // ]);
    await this.runCliCommand([
      'sed',
      '-i',
      's:<!-- <env name="DB_CONNECTION" value="sqlite"/> -->:<env name="DB_CONNECTION" value="mysql" />\\\n<env name="DB_DATABASE" value="testing" />\\\n<env name="DB_USERNAME" value="root" />\\\n<env name="DB_PASSWORD" value="root" />\\\n<env name="DB_HOST" value="db" />\\\n<env name="DB_PORT" value="3306" />:g',
      'phpunit.xml',
    ]);

    await this.runCliCommand(['php', 'artisan', 'deploy:install', '--extras', this.extras.toString()]);
  }
}

DeployLaravel.prototype.addAuth = addAuth;
DeployLaravel.prototype.addExtras = addExtras;

module.exports = DeployLaravel;
