const Deploy = require("../deploy");
const render = require("../output");
var inquirer = require("inquirer");
const util = require("util");
const { readFile, writeFile } = require("fs");
const exec = util.promisify(require("child_process").exec);
const { whatNodeVersionDoYouWant, addNvmrc } = require("./addNvmrc");
const addLintConfig = require("./addLintConfig");
const addNextConfig = require("./addNextConfig");
const addMaterialUI = require("./addMaterialUI");
const addStorybook = require("./addStorybook");
const addTestingLibrary = require("./addTestingLibrary");
const addGitlabSettings = require("./addGitlabSettings");
const { youWantReduxWithThat, addRedux } = require("./addRedux");

/**
- [ ]  basic routes / auth
- [ ]  Recommended packages (axiox, redux, formik + yup)
 */

class DeployNext extends Deploy {
  constructor(props, installedPath) {
    super();
    this.props = props;
    this.installedPath = installedPath;
    this.dependencies = ["prop-types"];
    this.devDependencies = [];
  }

  async deploy() {
    await this.checkSiteName();
    await this.getSitesPath();
    // TODO: REMOVE FALSE BEFORE DEPLOY!!!!!!
    await this.checkSiteFolderStatus(false);
    await this.youWantReduxWithThat();
    await this.whatNodeVersionDoYouWant();

    render.arrow(`Creating Next App...`);
    await this.composeNext();
  }

  async composeNext() {
    // Create Next App
    await this.runProcess("npx", ["create-next-app", this.props.siteName, '--js', '--eslint'], {
      cmd: this.props.sitePath,
    });

    // prepare folders...
    await exec(`mkdir ${this.props.sitePath}/src`);
    await exec(`mkdir ${this.props.sitePath}/src/utils`);
    await exec(`mkdir ${this.props.sitePath}/src/components`);
    await exec(`mkdir ${this.props.sitePath}/src/containers`);
    await exec(`rm -r ${this.props.sitePath}/pages`);
    await exec(`rm -r ${this.props.sitePath}/styles`);

    // install config and libraries...
    await this.addNvmrc();
    await this.addLintConfig();
    await this.addNextConfig();
    await this.addMaterialUI();
    await this.addStorybook();
    await this.addTestingLibrary();
    await this.addGitlabSettings();

    if (this.props.redux) {
      await this.addRedux();
    }

    // install yarn dependencies
    await this.installDependancies();
  }

  async checkSiteName() {
    if (this.props.siteName === "") {
      const { siteName } = await inquirer.prompt({
        type: "propt",
        name: "siteName",
        message: "Please enter site name",
        validate: function (value) {
          var pass = value.match(/^[a-z0-9\-]*$/i);
          if (pass && value.length >= 3) {
            return true;
          }
          return "Please enter a valid site name (a-z / 0-9)";
        },
      });
      this.props.siteName = siteName;
    }
  }

  async updatePackageDotJsonScripts(scripts) {
    const packageDotJson = `${this.props.sitePath}/package.json`;

    readFile(packageDotJson, "utf-8", function (err, contents) {
      if (err) {
        throw err;
      }

      const packageJson = JSON.parse(contents);

      // save..
      writeFile(
        packageDotJson,
        JSON.stringify({
          ...packageJson,
          scripts: { ...packageJson.scripts, ...scripts },
        }),
        "utf-8",
        function (err) {
          if (err) {
            throw err;
          }
        }
      );
    });
  }

  /**
   * Prepare list of dependencies...
   */
  addDependencies(deps, dev = false) {
    deps.forEach((dep) => {
      if (dev) {
        this.devDependencies.push(dep);
      } else {
        this.dependencies.push(dep);
      }
    });
  }

  /**
   * Install dependencies...
   */
  async installDependancies() {
    render.arrow(`Installing dependencies...`);

    await this.runProcess("yarn", ["add", ...this.dependencies, "--silent"], {
      cwd: this.props.sitePath,
    });
    await this.runProcess(
      "yarn",
      ["add", ...this.devDependencies, "-D", "--silent"],
      {
        cwd: this.props.sitePath,
      }
    );
  }
}

DeployNext.prototype.addNvmrc = addNvmrc;
DeployNext.prototype.addLintConfig = addLintConfig;
DeployNext.prototype.addNextConfig = addNextConfig;
DeployNext.prototype.addMaterialUI = addMaterialUI;
DeployNext.prototype.addStorybook = addStorybook;
DeployNext.prototype.addTestingLibrary = addTestingLibrary;
DeployNext.prototype.addGitlabSettings = addGitlabSettings;
DeployNext.prototype.youWantReduxWithThat = youWantReduxWithThat;
DeployNext.prototype.whatNodeVersionDoYouWant = whatNodeVersionDoYouWant;
DeployNext.prototype.addRedux = addRedux;

module.exports = DeployNext;
