const render = require('./output');
const util = require('util');
const exec = util.promisify(require('child_process').exec);
var inquirer = require('inquirer');

exports.checkDependancies = async function(options) {
  render.header('Checking all Dependancies', true);

  await checkHomebrew();
  await checkDocker();
  await checkDockerCompose();
  await checkDocksal();
  await checkMkcert();
  await checkDockalGlobals();
};

async function checkHomebrew() {
  render.arrow('Checking that homebrew is installed');
  try {
    const res = await exec('which brew >/dev/null 2>&1');
  } catch (error) {
    const { install } = await inquirer.prompt({
      type: 'confirm',
      name: 'install',
      message: 'Homebrew is not installed, would you like to install it now?',
      default: true,
    });

    if (install) {
      try {
        render.note('Installing Homebrew ...');
        const { stdout, stderr } = await exec('/usr/bin/ruby -e "$(curl -fsSL https://raw.githubusercontent.com/Homebrew/install/master/install)"');
        if (stderr !== '') {
          throw new Error(stderr);
        }
      } catch (error) {
        render.error(error);
        process.exit(1);
      }
    } else {
      render.error('could not install Brew');
      process.exit(1);
    }
  }
  const { stdout, stderr } = await exec('brew --version | head -n 1');
  render.success('\t' + stdout.trim() + ' found', true);
}

async function checkDocker() {
  render.arrow('Checking that Docker is installed');
  try {
    const res = await exec('which docker >/dev/null 2>&1');
  } catch (error) {
    render.error('Docker not found. Please install from https://hub.docker.com/');
    process.exit(1);
  }
  const { stdout, stderr } = await exec('docker --version | head -n 1');
  render.success('\t' + stdout.trim() + ' found', true);
}

async function checkDockerCompose() {
  render.arrow('Checking that Docker Compose is installed');
  try {
    const res = await exec('which docker-compose >/dev/null 2>&1');
  } catch (error) {
    render.error('Docker Compose not found. Please install from https://hub.docker.com/');
    process.exit(1);
  }
  const { stdout, stderr } = await exec('docker-compose --version | head -n 1');
  render.success('\t' + stdout.trim() + ' found', true);
}

async function checkDocksal() {
  render.arrow('Checking that Docksal is installed');
  try {
    const res = await exec('which fin >/dev/null 2>&1');
  } catch (error) {
    const { install } = await inquirer.prompt({
      type: 'confirm',
      name: 'install',
      message: 'Docksal is not installed, would you like to install it now?',
      default: true,
    });

    if (install) {
      try {
        render.note('Installing Docksal ...');
        const { stdout, stderr } = await exec('DOCKER_NATIVE=1 bash <(curl -fsSL https://get.docksal.io)');
        if (stderr !== '') {
          throw new Error(stderr);
        }
      } catch (error) {
        render.error(error);
        process.exit(1);
      }
    } else {
      render.error('could not install Docksal');
      process.exit(1);
    }
  }
  const { stdout, stderr } = await exec('fin version | head -n 1');
  render.success('\t' + stdout.trim() + ' found', true);
}

async function checkMkcert() {
  render.arrow('Checking that mkcert is installed');
  try {
    const res = await exec('which mkcert >/dev/null 2>&1');
  } catch (error) {
    const { install } = await inquirer.prompt({
      type: 'confirm',
      name: 'install',
      message: 'Mkcert is not installed, would you like to install it now?',
      default: true,
    });

    if (install) {
      try {
        render.note('Installing Mkcert ...');
        await exec('brew update');
        await exec('brew install mkcert nss');
        await exec('mkcert -install');
        await exec('mkdir ~/.docksal/certs/');
        if (stderr !== '') {
          throw new Error(stderr);
        }
      } catch (error) {
        render.error(error);
        process.exit(1);
      }
    } else {
      render.error('could not install mkcert');
      process.exit(1);
    }
  }
  const { stdout, stderr } = await exec('brew info mkcert | head -n 1');
  render.success('\t' + stdout.trim() + ' found', true);
}

async function checkDockalGlobals() {
  render.arrow('Checking that Docksal global packages are installed');
  try {
    await exec('which ~/.docksal/addons/artisan/artisan');
  } catch (error) {
    render.note('Installing artisan addon ...');
    await exec('fin addon install artisan --global');
  }
  render.success('artisan addon installed');

  try {
    await exec('which ~/.docksal/addons/mkcert/mkcert');
  } catch (error) {
    render.note('Installing mkcert addon ...');
    await exec('fin addon install mkcert --global');
  }
  render.success('mkcert addon installed');

  try {
    await exec('which ~/.docksal/addons/sequelace/sequelace');
  } catch (error) {
    render.note('Installing sequelace addon ...');
    await exec('fin addon install sequelace --global');
  }
  render.success('sequelace addon installed');

  try {
    await exec('which ~/.docksal/addons/tableplus/tableplus');
  } catch (error) {
    render.note('Installing tableplus addon ...');
    await exec('fin addon install tableplus --global');
  }
  render.success('tableplus addon installed');
}
