<?php

namespace App\Console\Commands;

use RuntimeException;
use Illuminate\Support\Arr;
use Illuminate\Support\Str;
use Illuminate\Console\Command;
use Illuminate\Support\Facades\File;
use Illuminate\Contracts\Filesystem\FileNotFoundException;

class Deploy extends Command
{
    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature = 'deploy:install {--extras=*}';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'Post deploy install commands';

    private $composerJson;

    /**
     * Create a new command instance.
     *
     * @return void
     */
    public function __construct()
    {
        parent::__construct();
    }

    /**
     * Execute the console command.
     *
     * @return void
     */
    public function handle(): void
    {
        $this->composerJson = $this->loadComposerJson();
        $this->ideHelper();

        if ($this->hasOption('extras')) {
            foreach ($this->option('extras') as $extra) {
                $extra = Str::slug($extra);
                $method = "{$extra}Setup";
                if (method_exists($this, $method)) {
                    $this->{$method}();
                }
            }
        }
        $this->sentrySetup();
        $this->phpcsfixerSetup();
        $this->saveComposerJson();

        $this->updateDatabaseConfig();
        $this->RolesPermissionSetup();
    }

    public function ideHelper()
    {
        $scripts = $this->composerJson['scripts'];
        if (! isset($scripts['post-update-cmd'])) {
            Arr::set($scripts, 'post-update-cmd', [
                "Illuminate\\Foundation\\ComposerScripts::postUpdate"
            ]);
        }
        $lines = [
            "Illuminate\\Foundation\\ComposerScripts::postUpdate",
            "@php artisan ide-helper:generate",
            "@php artisan ide-helper:meta",
            "@php artisan ide-helper:model -Nr"
        ];
        foreach ($lines as $line) {
            if (! in_array($line, $scripts['post-update-cmd'])) {
                $scripts['post-update-cmd'][] = $line;
            }
        }

        if (! isset($scripts['test'])) {
            Arr::set($scripts, 'test', [
                "@php artisan test --parallel",
            ]);
        }

        $this->composerJson['scripts'] = $scripts;
    }

    public function breezeSetup()
    {
        $this->call('breeze:install', ['stack' => 'api']);
    }

    public function sanctumSetup()
    {
        $file = base_path('app/Http/Kernel.php');
        $content = file_get_contents($file);
        if (! Str::contains($content, 'EnsureFrontendRequestsAreStateful::class')) {
            $content = str_replace(
                "'api' => [\n",
                "'api' => [\n            \Laravel\Sanctum\Http\Middleware\EnsureFrontendRequestsAreStateful::class,\n",
                $content
            );
            file_put_contents($file, $content);
        }

        $file = base_path('app/Models/User.php');
        $content = file_get_contents($file);
        if (! Str::contains($content, 'HasApiTokens')) {
            $content = str_replace(
                "use Illuminate\Notifications\Notifiable;",
                "use Illuminate\Notifications\Notifiable;\n    use Laravel\Sanctum\HasApiTokens;",
                $content
            );
            $content = str_replace(
                "class User extends Authenticatable\n{",
                "class User extends Authenticatable\n{\nuse HasApiTokens;",
                $content
            );
        }

        file_put_contents($file, $content);
    }

    public function jetstreamSetup()
    {
        $this->call('jetstream:install', ['stack' => 'livewire']);
    }

    public function passportSetup()
    {
        $this->call('passport:install');
        $file = base_path('app/Models/User.php');
        $content = file_get_contents($file);
        if (! Str::contains($content, 'HasApiTokens')) {
            $content = str_replace(
                "use Illuminate\Notifications\Notifiable;",
                "use Illuminate\Notifications\Notifiable;\n    use Laravel\Passport\HasApiTokens;",
                $content
            );
            $content = str_replace(
                "class User extends Authenticatable\n{",
                "class User extends Authenticatable\n{\nuse HasApiTokens;",
                $content
            );
        }
    }

    protected function getComposerJsonPath()
    {
        return base_path('composer.json');
    }

    protected function loadComposerJson()
    {
        $composerJsonPath = $this->getComposerJsonPath();

        if (! File::exists($composerJsonPath)) {
            throw new FileNotFoundException('composer.json does not exist');
        }

        $composerJsonContent = File::get($composerJsonPath);
        $composerJson = json_decode($composerJsonContent, true);

        if (! is_array($composerJson)) {
            throw new RuntimeException("Invalid composer.json file [$composerJsonPath]");
        }

        return $composerJson;
    }

    protected function saveComposerJson()
    {
        $newComposerJson = json_encode(
            $this->composerJson,
            JSON_PRETTY_PRINT | JSON_UNESCAPED_SLASHES
        );

        $composerJsonPath = $this->getComposerJsonPath();
        if (File::put($composerJsonPath, $newComposerJson) === false) {
            throw new RuntimeException("Cannot write to composer.json [$composerJsonPath]");
        }
    }

    protected function larastanSetup()
    {
        $scripts = $this->composerJson['scripts'];
        if (! isset($scripts['phpstan'])) {
            Arr::set($scripts, 'phpstan', [
                "phpstan analyse --",
            ]);
        }

        $this->composerJson['scripts'] = $scripts;
    }

    protected function phpcsfixerSetup()
    {
        $scripts = $this->composerJson['scripts'];
        if (! isset($scripts['phpcs'])) {
            Arr::set($scripts, 'phpcs', [
                "@php artisan fixer:fix --",
            ]);
        }
        $this->call('vendor:publish', ['--provider' => "STS\Fixer\FixerServiceProvider"]);
        $this->composerJson['scripts'] = $scripts;
    }

    public function updateDatabaseConfig()
    {

        $str = file_get_contents(config_path('database.php'));

        $content = "
            'driver' => 'mysql',
            'url' => env('DATABASE_URL'),
            'host' => env('DB_HOST', '127.0.0.1'),";
        $contentB = "
            'driver' => 'mysql',
            'url' => env('DB_URL'),
            'host' => env('DB_HOST', '127.0.0.1'),";
        $replace = "
            'driver' => 'mysql',
             'read' => [
                'host' => [
                    env('DB_HOST', '127.0.0.1'),
                ],
            ],
            'write' => [
                'host' => [
                    env('DB_READER_HOST', env('DB_HOST', '127.0.0.1')),
                ],
            ],
            'sticky' => true,";

        $str = str_replace([$content, $contentB], $replace, $str);
        file_put_contents(config_path('database.php'), $str);
    }

    public function RolesPermissionSetup()
    {
        $file = base_path('app/Models/User.php');
        $content = file_get_contents($file);
        if (! Str::contains($content, 'use Spatie\Permission\Traits\HasRoles;')) {
            $content = str_replace(
                "use Illuminate\Notifications\Notifiable;",
                "use Illuminate\Notifications\Notifiable;\n    use Spatie\Permission\Traits\HasRoles;",
                $content
            );
            $content = str_replace(
                "class User extends Authenticatable\n{",
                "class User extends Authenticatable\n{\nuse HasRoles;",
                $content
            );
            file_put_contents($file, $content);
        }
    }

    protected function sentrySetup()
    {
        $file = base_path('config/logging.php');
        $content = file_get_contents($file);
        $content = Str::replace("['single']", "['single','slack']", $content);
        $content = Str::replace("'papertrail'", "'sentry'     => [
            'driver' => 'sentry',
        ],\n'papertrail'", $content);
        file_put_contents($file, $content);
    }
}
